@@ -136,8 +136,8 @@ humanlogtime(void)
 		 tm->tm_hour, tm->tm_min, tm->tm_sec);
 #else
 	/* ISO 8601 is a better format, sort order equals time order */
-	snprintf(bp, LIB_BUFLENGTH, "%02d-%02dT%02d:%02d:%02d",
-		 tm->tm_mon+1, tm->tm_mday,
+	snprintf(bp, LIB_BUFLENGTH, "%04d-%02d-%02dT%02d:%02d:%02d",
+		 tm->tm_year+1900, tm->tm_mon+1, tm->tm_mday,
 		 tm->tm_hour, tm->tm_min, tm->tm_sec);
 #endif /* ENABLE_CLASSIC_MODE */
 
