@@ -1472,14 +1472,14 @@ config_access(
 
 		case T_Incalloc:
 			if (0 <= my_opt->value.i)
-				mru_incalloc = my_opt->value.u;
+				mon_data.mru_incalloc = my_opt->value.u;
 			else
 				range_err = true;
 			break;
 
 		case T_Incmem:
 			if (0 <= my_opt->value.i)
-				mru_incalloc = (my_opt->value.u * 1024U)
+				mon_data.mru_incalloc = (my_opt->value.u * 1024U)
 						/ sizeof(mon_entry);
 			else
 				range_err = true;
@@ -1487,14 +1487,14 @@ config_access(
 
 		case T_Initalloc:
 			if (0 <= my_opt->value.i)
-				mru_initalloc = my_opt->value.u;
+				mon_data.mru_initalloc = my_opt->value.u;
 			else
 				range_err = true;
 			break;
 
 		case T_Initmem:
 			if (0 <= my_opt->value.i)
-				mru_initalloc = (my_opt->value.u * 1024U)
+				mon_data.mru_initalloc = (my_opt->value.u * 1024U)
 						 / sizeof(mon_entry);
 			else
 				range_err = true;
@@ -1502,32 +1502,32 @@ config_access(
 
 		case T_Mindepth:
 			if (0 <= my_opt->value.i)
-				mru_mindepth = my_opt->value.u;
+				mon_data.mru_mindepth = my_opt->value.u;
 			else
 				range_err = true;
 			break;
 
 		case T_Maxage:
-			mru_maxage = my_opt->value.i;
+			mon_data.mru_maxage = my_opt->value.i;
 			break;
 
 		case T_Minage:
-			mru_minage = my_opt->value.i;
+			mon_data.mru_minage = my_opt->value.i;
 			break;
 
 		case T_Maxdepth:
 			if (0 <= my_opt->value.i)
-				mru_maxdepth = my_opt->value.u;
+				mon_data.mru_maxdepth = my_opt->value.u;
 			else
-				mru_maxdepth = UINT_MAX;
+				mon_data.mru_maxdepth = UINT_MAX;
 			break;
 
 		case T_Maxmem:
 			if (0 <= my_opt->value.i)
-				mru_maxdepth = (my_opt->value.u * 1024U) /
+				mon_data.mru_maxdepth = (my_opt->value.u * 1024U) /
 					       sizeof(mon_entry);
 			else
-				mru_maxdepth = UINT_MAX;
+				mon_data.mru_maxdepth = UINT_MAX;
 			break;
 
 		default:
@@ -1563,7 +1563,7 @@ config_access(
 			break;
 
 		case T_Monitor:
-			mon_age = my_opt->value.i;
+			mon_data.mon_age = my_opt->value.i;
 			break;
 
 		default:
@@ -2239,11 +2239,11 @@ config_mdnstries(
 	config_tree *ptree
 	)
 {
-#if defined(HAVE_DNS_SD_H) && defined(ENABLE_MDNS_REGISTRATION)
+#if defined(HAVE_DNS_SD_H)
 	mdnstries = ptree->mdnstries;
 #else
 	UNUSED_ARG(ptree);
-#endif  /* ENABLE_MDNS_REGISTRATION */
+#endif  /* HAVE_DNS_SD_H */
 }
 
 static void
@@ -2484,9 +2484,10 @@ config_vars(
 
 #ifdef ENABLE_LEAP_SMEAR
 		case T_Leapsmearinterval:
-			if (curr_var->value.i < 0)
+			if (curr_var->value.i < 0) {
 				msyslog(LOG_ERR, "CONFIG: negative leap smear interval ignored: %i", curr_var->value.i);
 				break;
+			}
 			leap_smear_intv = curr_var->value.u;
 			msyslog(LOG_INFO, "CONFIG: leap smear interval %u sec", leap_smear_intv);
 			break;
@@ -2615,7 +2616,7 @@ peer_config(
 	 * are.
 	 */
 	ctl->flags |= FLAG_CONFIG;
-	if (mode_ntpdate)
+	if (clock_ctl.mode_ntpdate)
 		ctl->flags |= FLAG_IBURST;
 	return newpeer(srcadr, hostname, dstadr, hmode,
 		       ctl, cast_flags, true);
