@@ -7,7 +7,7 @@
  * This is an attempt to get the leap second handling into a dedicated
  * module to make the somewhat convoluted logic testable.
  *
- * Note: Ths code assumes that the unsigned long long return value of
+ * Note: This code assumes that the unsigned long long return value of
  * strtoull(3) is large enough to parse any integer literal found in these
  * files, and that C will promote such values to uint64_t properly.
  *
@@ -76,7 +76,7 @@ static bool  _electric;
 /* Forward decls of local helpers */
 static bool   add_range(leap_table_t*, const leap_info_t*);
 static char * get_line(leapsec_reader, void*, char*, size_t);
-static char * skipws(char*);
+static char * skipws(char*) __attribute__((pure));
 static bool   parsefail(const char * cp, const char * ep);
 static void   reload_limits(leap_table_t*, time_t);
 static void   reset_times(leap_table_t*);
@@ -191,20 +191,20 @@ leapsec_load(
 				ntp = strtoull(cp, &ep, 10);
 				if (parsefail(cp, ep))
 					goto fail_read;
-				pt->head.expire = ntp - JAN_1970;
+				pt->head.expire = (time_t)(ntp - JAN_1970);
 				pt->lsig.etime = pt->head.expire;
 			} else if (*cp == '$') {
 				cp = skipws(cp+1);
 				ntp = strtoull(cp, &ep, 10);
 				if (parsefail(cp, ep))
 					goto fail_read;
-				pt->head.update = ntp - JAN_1970;
+				pt->head.update = (time_t)(ntp - JAN_1970);
 			}
 		} else if (isdigit((uint8_t)*cp)) {
 			ntp = strtoull(cp, &ep, 10);
 			if (parsefail(cp, ep))
 				goto fail_read;
-			ttime = ntp - JAN_1970;
+			ttime = (time_t)(ntp - JAN_1970);
 			cp = skipws(ep);
 			taiof = strtol(cp, &ep, 10);
 			if (   parsefail(cp, ep)
@@ -294,7 +294,7 @@ leapsec_query(
 		 */
 		last = pt->head.ttime;
 		// FIXME warped is only 16 bits.  ????
-		qr->warped = (uint16_t)(last - pt->head.dtime);
+		qr->warped = (int16_t)(last - pt->head.dtime);
 		next = when + qr->warped;
 		reload_limits(pt, next);
 		fired = (pt->head.ebase == last);
@@ -314,7 +314,7 @@ leapsec_query(
 	/* now start to collect the remaining data */
 	qr->tai_diff  = pt->head.next_tai - pt->head.this_tai;
 	qr->ttime     = pt->head.ttime;
-	qr->ddist     = pt->head.dtime - when;
+	qr->ddist     = (uint32_t)(pt->head.dtime - when);
 	qr->dynamic   = pt->head.dynls;
 
 	qr->proximity = LSPROX_SCHEDULE;
@@ -385,24 +385,24 @@ leapsec_load_stream(
 		switch (rcheck)
 		{
 		case LSVALID_GOODHASH:
-			msyslog(LOG_NOTICE, "%s ('%s'): good hash signature",
+			msyslog(LOG_NOTICE, "CLOCK: %s ('%s'): good hash signature",
 				logPrefix, fname);
 			break;
 			
 		case LSVALID_NOHASH:
-			msyslog(LOG_ERR, "%s ('%s'): no hash signature",
+			msyslog(LOG_ERR, "CLOCK: %s ('%s'): no hash signature",
 				logPrefix, fname);
 			break;
 		case LSVALID_BADHASH:
-			msyslog(LOG_ERR, "%s ('%s'): signature mismatch",
+			msyslog(LOG_ERR, "CLOCK: %s ('%s'): signature mismatch",
 				logPrefix, fname);
 			break;
 		case LSVALID_BADFORMAT:
-			msyslog(LOG_ERR, "%s ('%s'): malformed hash signature",
+			msyslog(LOG_ERR, "CLOCK: %s ('%s'): malformed hash signature",
 				logPrefix, fname);
 			break;
 		default:
-			msyslog(LOG_ERR, "%s ('%s'): unknown error code %d",
+			msyslog(LOG_ERR, "CLOCK: %s ('%s'): unknown error code %d",
 				logPrefix, fname, rcheck);
 			break;
 		}
@@ -414,15 +414,15 @@ leapsec_load_stream(
 	if (!leapsec_load(pt, (leapsec_reader)getc, ifp)) {
 		switch (errno) {
 		case EINVAL:
-			msyslog(LOG_ERR, "%s ('%s'): bad transition time",
+			msyslog(LOG_ERR, "CLOCK: %s ('%s'): bad transition time",
 				logPrefix, fname);
 			break;
 		case ERANGE:
-			msyslog(LOG_ERR, "%s ('%s'): times not ascending",
+			msyslog(LOG_ERR, "CLOCK: %s ('%s'): times not ascending",
 				logPrefix, fname);
 			break;
 		default:
-			msyslog(LOG_ERR, "%s ('%s'): parsing error",
+			msyslog(LOG_ERR, "CLOCK: %s ('%s'): parsing error",
 				logPrefix, fname);
 			break;
 		}
@@ -430,12 +430,12 @@ leapsec_load_stream(
 	}
 
 	if (pt->head.size)
-		msyslog(LOG_NOTICE, "%s ('%s'): loaded, expire=%s last=%s ofs=%d",
+		msyslog(LOG_NOTICE, "CLOCK: %s ('%s'): loaded, expire=%s last=%s ofs=%d",
 			logPrefix, fname, rfc3339time(pt->head.expire),
 			rfc3339time(pt->info[0].ttime), pt->info[0].taiof);
 	else
 		msyslog(LOG_NOTICE,
-			"%s ('%s'): loaded, expire=%s ofs=%d (no entries after build date)",
+			"CLOCK: %s ('%s'): loaded, expire=%s ofs=%d (no entries after build date)",
 			logPrefix, fname, rfc3339time(pt->head.expire),
 			pt->head.base_tai);
 	
@@ -459,9 +459,10 @@ leapsec_load_file(
 		return false;
 	
 	/* try to stat the leapfile */
+	/* coverity[toctou] */
 	if (0 != stat(fname, &sb_new)) {
 		if (logall)
-			msyslog(LOG_ERR, "%s ('%s'): stat failed: %m",
+			msyslog(LOG_ERR, "CLOCK: %s ('%s'): stat failed: %m",
 				logPrefix, fname);
 		return false;
 	}
@@ -496,7 +497,7 @@ leapsec_load_file(
 	if ((fp = fopen(fname, "r")) == NULL) {
 		if (logall)
 			msyslog(LOG_ERR,
-				"%s ('%s'): open failed: %m",
+				"CLOCK: %s ('%s'): open failed: %m",
 				logPrefix, fname);
 		return false;
 	}
@@ -905,7 +906,7 @@ do_hash_data(
 	unsigned int   tlen =  0;
 	unsigned char  ch;
 
-	while ('\0' != (ch = *cp++) && '#' != ch)
+	while ('\0' != (ch = (unsigned char)(*cp++)) && '#' != ch)
 		if (isdigit(ch)) {
 			text[tlen++] = ch;
 			tlen &= (sizeof(text)-1);
